Extension { #name : 'AbstractCharacter' }

{ #category : 'Non-Printable Characters' }
AbstractCharacter class >> backspace [

"(Subclass responsibility.)  Returns the back-space character from the
 receiver's character set."

AbstractCharacter subclassResponsibility: #backspace

]

{ #category : 'Non-Printable Characters' }
AbstractCharacter class >> cr [

"(Subclass responsibility.)  Returns the carriage-return character from the
 receiver's character set."

AbstractCharacter subclassResponsibility: #cr

]

{ #category : 'Non-Printable Characters' }
AbstractCharacter class >> esc [

"(Subclass responsibility.)  Returns the escape character from the receiver's
 character set."

AbstractCharacter subclassResponsibility: #esc

]

{ #category : 'Instance Creation' }
AbstractCharacter class >> fromStream: aStream [

"(Subclass responsibility.)  Returns an instance of the receiver's class that is
 the next character in the stream, aStream."

AbstractCharacter subclassResponsibility: #fromStream:

]

{ #category : 'Non-Printable Characters' }
AbstractCharacter class >> lf [

"(Subclass responsibility.)  Returns the line-feed character from the receiver's
 character set."

AbstractCharacter subclassResponsibility: #lf

]

{ #category : 'Non-Printable Characters' }
AbstractCharacter class >> newPage [

"(Subclass responsibility.)  Returns the new-page character from the receiver's
 character set."

AbstractCharacter subclassResponsibility: #newPage

]

{ #category : 'Non-Printable Characters' }
AbstractCharacter class >> space [

"(Subclass responsibility.)  Returns the space character from the receiver's
 character set."

AbstractCharacter subclassResponsibility: #space

]

{ #category : 'Non-Printable Characters' }
AbstractCharacter class >> tab [

"(Subclass responsibility.)  Returns the tab character from the receiver's
 character set."

AbstractCharacter subclassResponsibility: #tab

]

{ #category : 'Instance Creation' }
AbstractCharacter class >> withValue: anInteger [

"(Subclass responsibility.)  Returns an instance of the receiver's class that
 has the specified internal value."

AbstractCharacter subclassResponsibility: #withValue:

]

{ #category : 'New Indexing Comparison' }
AbstractCharacter >> _classSortOrdinal [

^ 30

]

{ #category : 'Indexing Support' }
AbstractCharacter >> _idxBasicCanCompareWithClass: aClass [
  "Returns true if the receiver may be inserted into a basic BtreeNode whose
   #lastElementClass is <aClass> (see RangeEqualityIndex class>>isBasicClass:)."

  (super _idxBasicCanCompareWithClass: aClass)
    ifTrue: [ ^ true ].
  ^ aClass isSubclassOf: AbstractCharacter

]

{ #category : 'Comparing' }
AbstractCharacter >> <= aCharacter [

"Returns true if the receiver is less than or equal to the argument,
 otherwise returns false."

^ (self < aCharacter) or: [self = aCharacter]

]

{ #category : 'Comparing' }
AbstractCharacter >> > aCharacter [

"Returns true if the receiver is greater than the argument,
 otherwise returns false."

^ (self <= aCharacter) not

]

{ #category : 'Comparing' }
AbstractCharacter >> >= aCharacter [

"Returns true if the receiver is greater than the argument,
 otherwise returns false."

^ (self < aCharacter) not

]

{ #category : 'Converting' }
AbstractCharacter >> asCharacter [

"(Subclass responsibility.)  Returns the Character corresponding to the
 receiver."

AbstractCharacter subclassResponsibility: #asCharacter

]

{ #category : 'Deprecated' }
AbstractCharacter >> asciiLessThan: aChar [

"Returns true if the ASCII code of the receiver is less than that of
 aCharacter."

self deprecated: 'asciiLessThan: deprecated in 3.2.'.
^ self codePoint < aChar codePoint

]

{ #category : 'Converting' }
AbstractCharacter >> asInteger [

"(Subclass responsibility.)  Returns the internal code of the receiver."

AbstractCharacter subclassResponsibility: #asInteger

]

{ #category : 'Converting' }
AbstractCharacter >> asLowercase [

"(Subclass responsibility.)  Returns a kind of AbstractCharacter that is the
 lower-case character corresponding to the receiver.  If the receiver
 is lower-case or has no case, this returns the receiver itself."

AbstractCharacter subclassResponsibility: #asLowercase

]

{ #category : 'Converting' }
AbstractCharacter >> asUppercase [

"(Subclass responsibility.)  Returns a kind of AbstractCharacter that is the
 upper-case character corresponding to the receiver.  If the receiver
 is upper-case or has no case, this returns the receiver itself."

AbstractCharacter subclassResponsibility: #asUppercase

]

{ #category : 'Converting' }
AbstractCharacter >> digitValue [

"(Subclass responsibility.)  Returns a SmallInteger representing the value of
 the receiver, a digit, or returns nil if the receiver is not a digit."

AbstractCharacter subclassResponsibility: #digitValue

]

{ #category : 'Formatting' }
AbstractCharacter >> displayWidth [

"(Subclass responsibility.)  Returns the width necessary to display the
 receiver."

AbstractCharacter subclassResponsibility: #displayWidth

]

{ #category : 'Comparing' }
AbstractCharacter >> hash [

"Returns a numeric hash key for the receiver."

^self asInteger

]

{ #category : 'Testing' }
AbstractCharacter >> isDigit [

"(Subclass responsibility.)  Returns true if the receiver is a digit.  Returns
 false otherwise."

AbstractCharacter subclassResponsibility: #isDigit

]

{ #category : 'Testing' }
AbstractCharacter >> isEquivalent: aCharacter [

"(Subclass responsibility.)  Returns true if the receiver is equivalent to
 aCharacter.  The receiver is equivalent to aCharacter if the receiver is the
 same character as the argument regardless of case or internal representation."

AbstractCharacter subclassResponsibility: #isEquivalent:

]

{ #category : 'Testing' }
AbstractCharacter >> isLowercase [

"(Subclass responsibility.)  Returns true if the receiver is a lower-case
 character.  Returns false otherwise."

AbstractCharacter subclassResponsibility: #isLowercase

]

{ #category : 'Testing' }
AbstractCharacter >> isUppercase [

"(Subclass responsibility.)  Returns true if the receiver is an upper-case
 character.  Returns false otherwise."

AbstractCharacter subclassResponsibility: #isUppercase

]
